/*
 * Copyright 2002 Sun Microsystems Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <libwnck/libwnck.h>
#include <gtk/gtk.h>
#include <gtk/gtkwidget.h>
#include <errno.h>
#include <unistd.h>
#include "pager-accessible.h"
#include "pager-accessible-factory.h"

static void                  wnck_pager_accessible_class_init      (WnckPagerAccessibleClass *klass);
static G_CONST_RETURN gchar* wnck_pager_accessible_get_name        (AtkObject                *obj);
static G_CONST_RETURN gchar* wnck_pager_accessible_get_description (AtkObject                *obj);

static gint       wnck_pager_accessible_get_n_children (AtkObject *obj);
static AtkObject* wnck_pager_accessible_ref_child      (AtkObject *obj, 
		                                        gint       i);

static void atk_selection_interface_init (AtkSelectionIface *iface);

static gboolean   wnck_pager_add_selection             (AtkSelectionIface *selection, 
		                                        gint               i);
static gboolean   wnck_pager_is_child_selected         (AtkSelection *selection, 
		                                        gint          i);
static AtkObject* wnck_pager_ref_selection             (AtkSelection *selection, 
		                                        gint          i);
static gint       wnck_pager_selection_count           (AtkSelection *selection);
static void       wnck_pager_accessible_init_workspace (AtkObject *aobj_ws, 
		                                        gint       i);


extern void           wnck_pager_activate_workspace   (WnckWorkspace *wspace);
extern gint           wnck_pager_get_n_ws             (WnckPager     *pager);
extern gchar*         wnck_pager_get_ws_name          (gint           i);
extern WnckWorkspace* wnck_pager_get_active_workspace (WnckPager     *pager);



/*
 * Array for holding the AtkObject structures for the children - Work Spaces
 */
AtkObject *aobj_ws[GET_MAX_N_WS];

GType
wnck_pager_accessible_get_type (void)
{
  static GType type = 0;

  if (!type) 
    {
      static GTypeInfo tinfo = 
      {
        sizeof (WnckPagerAccessibleClass),
        (GBaseInitFunc) NULL, /* base init */
        (GBaseFinalizeFunc) NULL, /* base finalize */
        (GClassInitFunc) wnck_pager_accessible_class_init, /* class init */
        (GClassFinalizeFunc) NULL, /* class finalize */
        NULL, /* class data */
        sizeof (WnckPagerAccessible), /* instance size */
        0, /* nb preallocs */
        NULL, /* instance init */
        NULL /* value table */
      };

      static const GInterfaceInfo atk_selection_info = 
      {
        (GInterfaceInitFunc) atk_selection_interface_init,
        (GInterfaceFinalizeFunc) NULL,
        NULL
      };

      /*
       * Figure out the size of the class and instance
       * we are deriving from
       */
      AtkObjectFactory *factory;
      GType derived_type;
      GTypeQuery query;
      GType derived_atk_type;

      derived_type = g_type_parent (WNCK_TYPE_PAGER);  

      factory = atk_registry_get_factory (atk_get_default_registry(),
                                          derived_type);
      derived_atk_type = atk_object_factory_get_accessible_type (factory);
      g_type_query (derived_atk_type, &query);
      tinfo.class_size = query.class_size;
      tinfo.instance_size = query.instance_size;

      type = g_type_register_static (derived_atk_type,
                                     "WnckPagerAccessible", &tinfo, 0);

      g_type_add_interface_static (type, ATK_TYPE_SELECTION, &atk_selection_info);
    }
    
  return type;
}


static void
atk_selection_interface_init (AtkSelectionIface *iface)
{
  g_return_if_fail (iface != NULL);
  
  iface->add_selection = wnck_pager_add_selection;
  iface->ref_selection = wnck_pager_ref_selection;
  iface->get_selection_count = wnck_pager_selection_count;
  iface->is_child_selected = wnck_pager_is_child_selected;
}

static gboolean
wnck_pager_add_selection (AtkSelectionIface *selection, gint i)
{
  WnckPager *pager;
  WnckWorkspace *wspace;
  GtkWidget *widget;
  gint n_spaces; 

	n_spaces = GET_MAX_N_WS;

  widget = GTK_ACCESSIBLE (selection)->widget;
  if (widget == NULL) 
    {
      /* 
       *State is defunct
       */
      return FALSE;
    }
  pager = WNCK_PAGER (widget);
  
  if (i < 0 || i >= n_spaces)
    return FALSE;
    
  /*
   * Activate the following worksapce as current workspace
   */
  wspace = wnck_workspace_get (i); 
  wnck_pager_activate_workspace (wspace);
 
  return TRUE;
}

/*
 *Returns the AtkObject of the selected WorkSpace
 */
static AtkObject*
wnck_pager_ref_selection (AtkSelection *selection, gint i)
{
  WnckPager *pager;
  GtkWidget *widget;
  WnckWorkspace *active_wspace;
  AtkObject *accessible;
  gint wsno;       

  g_return_val_if_fail (i == 0, NULL);
       
  widget = GTK_ACCESSIBLE (selection)->widget;
  if (widget == NULL) 
    {
      /*
       * State is defunct
       */
      return FALSE;
    }
  pager = WNCK_PAGER (widget);
  
  active_wspace = WNCK_WORKSPACE (wnck_pager_get_active_workspace (pager));
  wsno = wnck_workspace_get_number (active_wspace);

  accessible = ATK_OBJECT (wnck_pager_accessible_ref_child (ATK_OBJECT (selection), wsno));
  
  return accessible;
}

/*
 * Returns the no.of child selected, it should be either 1 or 0
 * b'coz only one child can be selected at a time.
 */
static gint
wnck_pager_selection_count (AtkSelection *selection)
{
  WnckPager *pager;
  GtkWidget *widget;

  widget = GTK_ACCESSIBLE (selection)->widget;
  if (widget == NULL) 
    {
      /*
       * State is defunct
       */
      return FALSE;
    }
  pager = WNCK_PAGER (widget);
  if (pager == NULL) 
    {
      return 0;
    }
  else 
    {
       return 1;
    }
}

/*
 *Checks whether the WorkSpace specified by i is selected,
 *and returns TRUE on selection.
 */
static gboolean
wnck_pager_is_child_selected (AtkSelection *selection, gint i)
{
  WnckPager *pager;
  GtkWidget *widget;
  WnckWorkspace *active_wspace;
  gint wsno;

  widget = GTK_ACCESSIBLE (selection)->widget;
  if (widget == NULL) 
    {
      /*
       * State is defunct
       */
      return FALSE;
    }
  pager = WNCK_PAGER (widget);
  active_wspace = WNCK_WORKSPACE (wnck_pager_get_active_workspace (pager));

  wsno = wnck_workspace_get_number (active_wspace);
  
  return (wsno == i);
}

static void
wnck_pager_accessible_class_init (WnckPagerAccessibleClass *klass)
{
  AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
  
  class->get_name = wnck_pager_accessible_get_name;
  class->get_description = wnck_pager_accessible_get_description;
  class->get_n_children = wnck_pager_accessible_get_n_children;
  class->ref_child = wnck_pager_accessible_ref_child;
}

AtkObject*
wnck_pager_accessible_new (GtkWidget *widget)
{
  GObject *object;
  AtkObject *aobj_pager;

  GtkAccessible *gtk_accessible;

  object = g_object_new (WNCK_PAGER_TYPE_ACCESSIBLE, NULL);

  aobj_pager = ATK_OBJECT (object);

  gtk_accessible = GTK_ACCESSIBLE (aobj_pager);
  gtk_accessible->widget = widget;

  atk_object_initialize (aobj_pager, widget);
  aobj_pager->role = ATK_ROLE_PANEL;

  return aobj_pager;
  
}

static G_CONST_RETURN gchar*
wnck_pager_accessible_get_name (AtkObject *obj)
{
  g_return_val_if_fail (WNCK_PAGER_IS_ACCESSIBLE (obj), NULL);

  if (obj->name == NULL)
    {
      obj->name = g_strdup ("Workspace Switcher");
    }
  return obj->name;

}

static G_CONST_RETURN gchar*
wnck_pager_accessible_get_description (AtkObject *obj) 
{
  g_return_val_if_fail (WNCK_PAGER_IS_ACCESSIBLE (obj), NULL);

  if (obj->description == NULL)
    {
      obj->description = g_strdup ("Tool to switch between work spaces");
    }
  return obj->description;
  
}

/*
 * Number of WorkSpaces is returned as the n_children
 */
static gint
wnck_pager_accessible_get_n_children (AtkObject* obj)
{

  GtkAccessible *accessible;
  GtkWidget *widget;

  WnckPager *pager;
  gint n_spaces = 0;

  g_return_val_if_fail (WNCK_PAGER_IS_ACCESSIBLE (obj), n_spaces);
  accessible = GTK_ACCESSIBLE (obj);
  widget = accessible->widget;

  if (widget == NULL)
    /* State is defunct */
    return n_spaces;

  pager = WNCK_PAGER (widget);

  n_spaces = wnck_pager_get_n_ws (pager);

  return n_spaces;
  
}


/*
 *Will return Appropriate static AtkObject for the workspaces 
 */
static AtkObject*
wnck_pager_accessible_ref_child (AtkObject *obj,
         gint i)
{

  GtkAccessible *accessible;
  GtkWidget *widget;
  
  WnckPager *pager;
  WnckWorkspace *wspace;

  AtkRegistry *default_registry;   
  AtkObjectFactory *factory;

  gint n_spaces = 0;

  g_return_val_if_fail (WNCK_PAGER_IS_ACCESSIBLE (obj), NULL);
  g_return_val_if_fail (ATK_IS_OBJECT (obj), NULL);

  accessible = GTK_ACCESSIBLE (obj);
  widget = accessible->widget;

  if (widget == NULL)
    /* State is defunct */
    return NULL;

  pager = WNCK_PAGER (widget);
  
  n_spaces = wnck_pager_get_n_ws(pager); 

  if ( (i>=0) && (i < n_spaces) ) 
    {
      default_registry = atk_get_default_registry ();
      factory = atk_registry_get_factory (default_registry,
                                          WNCK_TYPE_WORKSPACE);
         
      wspace = WNCK_WORKSPACE (wnck_workspace_get (i));
      aobj_ws[i] = atk_object_factory_create_accessible (factory,
                                                         G_OBJECT(wspace));
      wnck_pager_accessible_init_workspace (aobj_ws[i], i);
      atk_object_set_parent (aobj_ws[i], obj);
  
      g_object_ref (aobj_ws[i]);

    }
  else 
    {
      return NULL;
    }
  return aobj_ws[i];

}

static void
wnck_pager_accessible_init_workspace (AtkObject *aobj_ws, gint i)
{
  aobj_ws->name = wnck_pager_get_ws_name (i);
  aobj_ws->description = g_strdup("Click on this to switch to this WorkSpace");
  aobj_ws->role = ATK_ROLE_UNKNOWN;
}
